function U = ell_liebmann
clear all;
close all;
%
%  Solve Laplace's equation using Liebmann's method
%
%  Assume Dirichlet Boundary Conditions
%
%  Author:  David Keffer
%  Department of Chemical Engineering
%  University of Tennessee, Knoxville
%  Last Updated:  October 24, 2000
%

%solve for several values of lambda
%lambdav = [1.0:0.1:1.9];
lambdav = [1.5];
for k = 1:1:length(lambdav)
   lambda = lambdav(k);
	% number of intervals
	nx_int = 20;
	ny_int = 20;
	% number of nodes
	nx = nx_int + 1;
	ny = ny_int + 1;
	%  grid points in x and y
	xo = 0.0;
	xf = 1.0;
	yo = 0.0;
	yf = 1.0;
	dx = (xf - xo)/nx_int;
	dy = (yf - yo)/ny_int;
	xgrid = [xo:dx:xf];
	ygrid = [yo:dy:yf];
	% initialize U and Uold
	U = zeros(nx,ny);
	Uold = zeros(nx,ny);
	%  Fill in Uold with four dirichlet BCs
	for j = 1:1:ny
   	% BC for x = xo
		Uold(1,j) = 0.0;
   	% BC for x = xf
   	Uold(nx,j) = ygrid(j)/20;
   	% BC for y = yo
		Uold(j,1) = 0.0;
   	% BC for y = yf
   	Uold(j,ny) = 0.05*sin(pi*xgrid(j)*0.5);
	end
	%  fill in the interior nodes of Uold with initial guess
	for i = 2:1:nx-1
   	for j = 2:1:ny-1
      	Uold(i,j) = 0.0;
	   end
	end
	U = Uold;
	%
	%  iteratively solve
	%
	maxit = 100;
	err = 100;
	tol = 1.0e-6;
	icount = 0;
	while (icount < maxit & err > tol)
   	icount = icount + 1;
   	Uold = U;
   	for i = 2:1:nx-1
         for j = 2:1:ny-1
            % apply Liebmann's method
         	U(i,j) = (U(i+1,j) + U(i-1,j) + U(i,j+1) + U(i,j-1) )*0.25;
         	U(i,j) = lambda*U(i,j) + (1-lambda)*Uold(i,j);
      	end
      end
      % calculate error
   	err = 0.0;
   	for i = 2:1:nx-1
   	   for j = 2:1:ny-1
   	      err = err + (U(i,j) - Uold(i,j))^2;
   	   end
   	end
   	err = sqrt(err/((nx-2)*(ny-2)));
	end
	fprintf(1,' lamdba = %f iteration = %i , error = %f \n',lambda, icount, err);
	% plot
	xmin = xgrid(1);
	xmax = xgrid(nx);
	ymin = ygrid(1);
	ymax = ygrid(ny);
	ncontourlines = 50;
	Umin = min(min(U));
	Umax = max(max(U));
	if (abs(Umin-Umax) < 1.0e-8);
      fprintf(1,'Solution is a flat plane with value = %f \n',Umin)
   else
      plot_dimensions = 3;
      if (plot_dimensions == 3) 
			contour3(xgrid, ygrid,U,ncontourlines);
   		axis([xmin xmax ymin ymax Umin Umax])
      else
         contour(xgrid, ygrid,U,ncontourlines);
   		axis([xmin xmax ymin ymax])
      end
      xlabel('x');
   	ylabel('y');
   	colorbar
	end	
end