function [xvec,tvec,Tmat] = hyperpde_n_anyBC
%
%  The routine hyperpde_n_anyBC will solve 
%  a system of non-linear 1-D hyperbolic PDEs of the form
%
%  dT/dt = f(x,t,T,dTdx,d2Tdx2)
%
%  using a second order method
%
%  The initial conditions must have the form
%  IC:  T(x,to,k_eq) = To(x,k_eq);
%
%  The boundary conditions can be
%  Dirichlet, Neumann or Mixed of the form
%  BC 1:  aBCo(t,k)*T(xo,t) + bBCo(t,k)*dTdx(xo,t) + cBCo(t,k) = 0;
%  BC 2:  aBCf(t,k)*T(xf,t) + bBCf(t,k)*dTdx(xf,t) + cBCf(t,k) = 0;
%
%  The necessary functions
%  pdefunk(x,t,T,dTdx,d2Tdx2,k_eq), To(x,k_eq),
%  aBCo(t,k), bBCo(t,k), cBCo(t,k), aBCf(t,k), bBCf(t,k), cBCf(t,k) 
%  are entered at the bottom of the file
%
%  The number of equations, neq, must be entered at the top of the file 
%
%  The initial value, final value and discretization in time, t, 
%  to, tf, and dt must be entered at the top of the file.
%
%  The initial value, final value and discretization in space, x,
%  xo, xf, and dx must be entered at the top of the file.
%
%  The type of boundary conditions 'D', 'N' or 'M'
%  for each boundary must be entered at the top of the file.
%
%  sample execution:
%  [xvec,tvec,Tmat] = hyperpde_n_anyBC;
%
%  code written by:  David J. Keffer
%  University of Tennessee, dkeffer@utk.edu
%  code written:  October 21, 2001
%  comments last updated:  February 26, 2014
%
clear all;
close all;
%  define number of PDEs
neq = 2;

%
%  define type of boundary condition
%  Choices are 'D' = Dirichlet, i.e. bBC(t) = 0
%  Choices are 'N' = Neumann, i.e. aBC(t) = 0
%  Choices are 'M' = Mixed, bBC(t) ~= 0 & aBC(t) ~= 0
%
%  The boundary is the first index.  The equation is second.
BC(1,1) = 'D';
BC(2,1) = 'D';
BC(1,2) = 'D';
BC(2,2) = 'D';


% discretize time
to = 0;
tf = 1.0e+0;
dt = 1.0e-3;
tvec = [to:dt:tf];
nt = length(tvec);

% discretize space
xo = 0;
xf = 1.0;
dx = 2.5e-2;
% include imaginary boundary nodes 
xvec = [xo-dx:dx:xf+dx];
nx = length(xvec);
dxi = 1.0/dx;

% dimension solution
Tmat = zeros(nx,nt,neq);


% dimension temporary vectors
Told = zeros(nx,1,neq);
Ttem = zeros(nx,1,neq);
Tnew = zeros(nx,1,neq);

% apply initial conditions to all real nodes
i = 1;
t = tvec(i);
for k = 1:1:neq
   for j = 2:1:nx-1
	  Tmat(j,i,k) = icfunk(xvec(j),k);
   end
end


% apply Neumann/Mixed BCs as ICs at imaginary nodes
for k = 1:1:neq
   if (BC(1,k) ~= 'D')
      Tmat(1,i,k) =  2.0*dx/bBCo(t,k)*( aBCo(t,k)*Tmat(2,i,k)    + bBCo(t,k)/(2.0*dx)*Tmat(3,i,k)    + cBCo(t,k)); 
   end
   if (BC(2,k) ~= 'D')
      Tmat(nx,i,k) = 2.0*dx/bBCf(t,k)*(-aBCf(t,k)*Tmat(nx-1,i,k) + bBCf(t,k)/(2.0*dx)*Tmat(nx-2,i,k) - cBCf(t,k));
   end
end
%
%  Determine, based on type of BC, how to define which nodes are determined by PDE vs BCs
%  ivaro = index of first node to be solved by PDE 
%  ivarf = index of last node to be solved by PDE 
%  nvar = number of nodes to be solved by PDE
%
for k = 1:1:neq
   if (BC(1,k) == 'D') 
      ivaro(k) = 3;
   else
      ivaro(k) = 2;
   end
   if (BC(2,k) == 'D') 
      ivarf(k) = nx-2;
   else
      ivarf(k) = nx-1;
   end
   nvar(k) = ivarf(k) - ivaro(k) + 1;
end


% loop over times
Told(1:nx,1:neq) = Tmat(1:nx,i,1:neq);
for i = 2:1:nt
%  update time
   t = tvec(i);

%
%  Prediction Step
%

%  compute first and second spatial derivatives
   for k = 1:1:neq
      for j = ivaro(k):1:ivarf(k)
         dTdx(j,k) = 0.5*( Told(j+1,k) - Told(j-1,k) )*dxi;
         d2Tdx2(j,k) = ( Told(j+1,k) - 2.0*Told(j,k) + Told(j-1,k) )*dxi^2;
      end
   end
   
% estimate slope  at beginning of temporal interval 
   for k = 1:1:neq
      for j = ivaro(k):1:ivarf(k)
         k1(j,k) = pdefunk(xvec(j),tvec(i),Told(j,1:neq),dTdx(j,1:neq),d2Tdx2(j,1:neq),k);
      end
   end

% apply Euler method for the prediction step
   for k = 1:1:neq
      for j = ivaro(k):1:ivarf(k)
         Ttem(j,k) = Told(j,k) + dt*k1(j,k);
      end
   end

%  apply BCs at the prediction step
   for k = 1:1:neq
      if (BC(1,k) == 'D') 
         Ttem(2,k) = -cBCo(t,k)/aBCo(t,k);
      else
         Ttem(1,k) =  2.0*dx/bBCo(t,k)*( aBCo(t,k)*Ttem(2,k)    + bBCo(t,k)/(2.0*dx)*Ttem(3,k)    + cBCo(t,k)); 
      end
      if (BC(2,k) == 'D') 
         Ttem(nx-1,k) = -cBCf(t,k)/aBCf(t,k);
      else
         Ttem(nx,k) = 2.0*dx/bBCf(t,k)*(-aBCf(t,k)*Ttem(nx-1,k) + bBCf(t,k)/(2.0*dx)*Ttem(nx-2,k) - cBCf(t,k));  
      end
   end
   
%
%  Correction Step
%

%  compute first and second spatial derivatives
   for k = 1:1:neq
      for j = ivaro(k):1:ivarf(k)
         dTdx(j,k) = 0.5*( Ttem(j+1,k) - Ttem(j-1,k) )*dxi;
         d2Tdx2(j,k) = ( Ttem(j+1,k) - 2.0*Ttem(j,k) + Ttem(j-1,k) )*dxi^2;
      end
   end

% estimate slope  at end of temporal interval 
   for k = 1:1:neq
      for j = ivaro(k):1:ivarf(k)
         k2(j,k) = pdefunk(xvec(j),tvec(i),Ttem(j,1:neq),dTdx(j,1:neq),d2Tdx2(j,1:neq),k);
      end
   end

% apply second-order method for the correction step
   for k = 1:1:neq
      for j = ivaro(k):1:ivarf(k)
         Tnew(j,k) = Told(j,k) + 0.50*dt*(k1(j,k)+k2(j,k));
      end
   end


%  apply BCs at the correction step
   for k = 1:1:neq
      if (BC(1,k) == 'D') 
         Tnew(2,k) = -cBCo(t,k)/aBCo(t,k);
      else
         Tnew(1,k) =  2.0*dx/bBCo(t,k)*( aBCo(t,k)*Tnew(2,k)    + bBCo(t,k)/(2.0*dx)*Tnew(3,k)    + cBCo(t,k)); 
      end
      if (BC(2,k) == 'D') 
         Tnew(nx-1,k) = -cBCf(t,k)/aBCf(t,k);
      else
         Tnew(nx,k) = 2.0*dx/bBCf(t,k)*(-aBCf(t,k)*Tnew(nx-1,k) + bBCf(t,k)/(2.0*dx)*Tnew(nx-2,k) - cBCf(t,k));  
      end
   end
   
% store new temperatures
   Tmat(1:nx,i,1:neq) = Tnew(1:nx,1:neq);
   Told(1:nx,1:neq) = Tnew(1:nx,1:neq);
end

% plot via movie
figure(1);
% number of frames in movie
nframe = 100;
% frames per second
fps = 5;
if (nt > nframe)
	nskip = round(nt/nframe);
else
   nskip = 1;
end
lplot = 1;
if (lplot == 1)
   fpsinv = 1.0/fps;
   newplot;
   Tmax=max(max(max(Tmat(2:nx-1,1:nt,1:neq))));
   Tmin=min(min(min(Tmat(2:nx-1,1:nt,1:neq))));
   xtext = xo + 0.1*(xf-xo);
   ytext = Tmin +0.9*(Tmax-Tmin);
   for i=1:nskip:nt
      for k = 1:1:neq
         %for j = 2:1:nx-1
         %   vector_plot(i_x) = T(k_eq,j_t,i_x);
         %end
		 color_index = get_plot_color(k);
         plot(xvec(2:nx-1) , Tmat(2:nx-1,i,k),color_index);
         hold on
      end
      axis([xo xf Tmin Tmax]);
      xlabel('position') 
      ylabel('vertical displacement or velocity') 
	  legend (int2str([1:neq]'));
      temps = char(strcat('time = ', num2str(tvec(i)), ' sec '));
      text(xtext, ytext,temps)
      hold off
      pause(fpsinv);
	end
end


%
%  functions defining PDE
%

function dydt_out = pdefunk(x,t,y,dydx,d2ydx2,keq);
c = 1.0;
dydt(1) = y(2);
dydt(2) = 1/c^2*d2ydx2(1);
dydt_out = dydt(keq);

%
%  function defining initial condition
%

function ic_out = icfunk(x,keq);
ic(1) = sin(x*2.0*pi);
ic(2) = 0.0;
ic_out = ic(keq);
  
%
%  functions defining LHS boundary condition
%  for hyperbolic BC(2) must be time derivative of BC(1)
%

function fout = aBCo(t,k);
f(1) = 1;
f(2) = 1;
fout = f(k);

function fout = bBCo(t,k);
f(1) = 0;
f(2) = 0;
fout = f(k);

function fout = cBCo(t,k);
f(1) = 0;
f(2) = 0;
fout = f(k);

%
%  functions defining RHS boundary condition
%  for hyperbolic BC(2) must be time derivative of BC(1)
%

function fout = aBCf(t,k);
f(1) = 1;
f(2) = 1;
fout = f(k);

function fout = bBCf(t,k);
f(1) = 0;
f(2) = 0;
fout = f(k);

function fout = cBCf(t,k);
f(1) = 0;
f(2) = 0;
fout = f(k);

























%
% this little function sets colors for curves in plot
%
function color_index = get_plot_color(i);
if (i == 1) 
   color_index = 'k-';
elseif (i == 2)
   color_index = 'r-';
elseif (i == 3)
   color_index = 'b-';
elseif (i == 4)
   color_index = 'g-';
elseif (i == 5)
   color_index = 'm-';
elseif (i == 6) 
   color_index = 'k:';
elseif (i == 7)
   color_index = 'r:';
elseif (i == 8)
   color_index = 'b:';
elseif (i == 9)
   color_index = 'g:';
elseif (i == 10)
   color_index = 'm:';
else
   color_index = 'k-';
end
