function [xvec,tvec,Tmat] = linparapde_crank_anyBC
%
%  The routine linparapde_crank_anyBC will solve 
%  a single linear 1-D parabolic PDE of the form
%
%  d(x,t)*dT/dt = div(c(x,t)*grad(T)) - a(x,t)*T -b(x,t)*dTdx +f(x,t)
%
%  using the Crank-Nicholson Method.
%
%  The initial condition must have the form
%  IC:  T(x,to) = To(x);
%
%  The boundary conditions can be
%  Dirichlet, Neumann or Mixed of the form
%  BC 1:  aBCo(t)*T(xo,t) + bBCo(t)*dTdx(xo,t) + cBCo(t) = 0;
%  BC 2:  aBCf(t)*T(xf,t) + bBCf(t)*dTdx(xf,t) + cBCf(t) = 0;
%
%  The necessary functions
%  a(x,t), b(x,t), c(x,t), dcdx(x,t), f(x,t), To(x),
%  aBCo(t), bBCo(t), cBCo(t), aBCf(t), bBCf(t), cBCf(t) 
%  are entered at the bottom of the file
%
%  The initial value, final value and discretization in time, t, 
%  to, tf, and dt must be entered at the top of the file.
%
%  The initial value, final value and discretization in space, x,
%  xo, xf, and dx must be entered at the top of the file.
%
%  The type of boundary conditions 'D', 'N' or 'M'
%  for each boundary must be entered at the top of the file.
%
%  sample execution:
%  [xvec,tvec,Tmat] = linparapde_crank_anyBC;
%
%  code written by:  David J. Keffer
%  University of Tennessee, dkeffer@utk.edu
%  code written:  October 21, 2001
%  comments last updated:  February 26, 2014
%
clear all;
close all;
%
%  define type of boundary condition
%  Choices are 'D' = Dirichlet, i.e. bBC(t) = 0
%  Choices are 'N' = Neumann, i.e. aBC(t) = 0
%  Choices are 'M' = Mixed, bBC(t) ~= 0 & aBC(t) ~= 0
%
BC(1) = 'D';
BC(2) = 'D';

% discretize time
to = 0;
tf = 4.0e+3;
dt = 4.0e+1;
tvec = [to:dt:tf];
nt = length(tvec);

% discretize space
xo = 0;
xf = 1.0;
dx = 1.0e-1;
% include imaginary boundary nodes 
xvec = [xo-dx:dx:xf+dx];
nx = length(xvec);

% dimension solution
Tmat = zeros(nx,nt);


% dimension temporary vectors
Told = zeros(nx,1);
Tnew = zeros(nx,1);
Amat = zeros(nx,nx);
bvec = zeros(nx,1);

% apply initial conditions to all real nodes
i = 1;
t = tvec(i);
for j = 2:1:nx-1
	x = xvec(j);
    Amat(j,j) = 1.0;
    bvec(j) = icfunk(x);
end

%  implement LHS BCs
   if (BC(1) == 'D')
      Amat(1,1) = 1.0;
      bvec(1) = 0.0d0;
%      Amat(2,2) = aBCo(t);
%      bvec(2) = -cBCo(t);
   else
      Amat(1,1) = -1.0/(2.0*dt)*bBCo(t);
      Amat(1,2) =               aBCo(t);
      Amat(1,3) =  1.0/(2.0*dt)*bBCo(t);
      bvec(1) = -cBCo(t);
   end

%  implement RHS BCs
   if (BC(2) == 'D')
      Amat(nx,nx) = 1.0;
      bvec(nx) = 0.0d0;
%      Amat(nx-1,nx-1) = aBCf(t);
%      bvec(nx-1) = -cBCf(t);
   else
      Amat(nx,nx-2) = -1.0/(2.0*dt)*bBCf(t);
      Amat(nx,nx-1) =               aBCf(t);
      Amat(nx,nx)   =  1.0/(2.0*dt)*bBCf(t);
      bvec(nx) = -cBCf(t);
   end

% store initial conditions in solution matrix
Tmat(1:nx,1) = inv(Amat)*bvec;

%
%  Determine, based on type of BC, how to define matrix
%  ivaro = index of first node to be solved by PDE 
%  ivarf = index of last node to be solved by PDE 
%  nvar = number of nodes to be solved by PDE
%
if (BC(1) == 'D') 
   ivaro = 3;
else
   ivaro = 2;
end
if (BC(2) == 'D') 
   ivarf = nx-2;
else
   ivarf = nx-1;
end
nvar = ivarf - ivaro + 1;

% loop over times
Told(1:nx) = Tmat(1:nx,i);
for i = 2:1:nt
%  reinitialize Amat and bvec
   Amat = zeros(nx,nx);
   bvec = zeros(nx,1);
%  update time
   told = tvec(i-1);
   t = tvec(i);
   
%  implement LHS BCs
   if (BC(1) == 'D')
      Amat(1,1) = 1.0;
      bvec(1) = 0.0d0;
      Amat(2,2) = aBCo(t);
      bvec(2) = -cBCo(t);
   else
      Amat(1,1) = -1.0/(2.0*dt)*bBCo(t);
      Amat(1,2) =               aBCo(t);
      Amat(1,3) =  1.0/(2.0*dt)*bBCo(t);
      bvec(1) = -cBCo(t);
   end

%  implement RHS BCs
   if (BC(2) == 'D')
      Amat(nx,nx) = 1.0;
      bvec(nx) = 0.0d0;
      Amat(nx-1,nx-1) = aBCf(t);
      bvec(nx-1) = -cBCf(t);
   else
      Amat(nx,nx-2) = -1.0/(2.0*dt)*bBCf(t);
      Amat(nx,nx-1) =               aBCf(t);
      Amat(nx,nx)   =  1.0/(2.0*dt)*bBCf(t);
      bvec(nx) = -cBCf(t);
   end
   
%  implement Crank-Nicholson for PDEs from ivaro to ivarf
   Kmatold = 0.5*getK(nx,ivaro,ivarf,told,xvec,dt,dx);
   Rvecold = 0.5*getR(nx,ivaro,ivarf,told,xvec,dt,dx);
   Kmatnew = 0.5*getK(nx,ivaro,ivarf,t,   xvec,dt,dx);
   Rvecnew = 0.5*getR(nx,ivaro,ivarf,t,   xvec,dt,dx);

%  add missing contributions to diagonal elements
   Kmatnew = -Kmatnew;
   for j = ivaro:1:ivarf
       Kmatnew(j,j) = 1.0 + Kmatnew(j,j);
       Kmatold(j,j) = 1.0 + Kmatold(j,j);
   end

%  build and invert Amat
       Amat(1:nx,1:nx) = Amat(1:nx,1:nx) + Kmatnew(1:nx,1:nx);
       Ainv = inv(Amat);
       
% build the bvec
       KmatTold = Kmatold*Told;
       for j = ivaro:1:ivarf
           bvec(j) = bvec(j) + KmatTold(j) + Rvecnew(j) + Rvecold(j);
       end
       
% solve for the new temperatures
   Tnew = Ainv*bvec;
   Tmat(1:nx,i) = Tnew;
   Told = Tnew;
end

% plot
figure(1);
nskip = 1;
for i = 1:nskip:nt
   plot(xvec(2:nx-1),Tmat(2:nx-1,i),'k-');
   %pause(1);
   hold on;
end
xlabel('position (m)')
ylabel('Temperature (K)');

function Kmat = getK(nx,ivaro,ivarf,t,xvec,dt,dx);
   Kmat = zeros(nx,nx);
   % diagonal elements of matrix
   for j = ivaro:1:ivarf
      x = xvec(j+1);
      Kmat(j,j) =  dt/dfunk(x,t)* (-2*cfunk(x,t)/dx^2 - afunk(x,t) );
   end
   % upper off-diagonal elements of matrix
   for j = ivaro:1:ivarf
      x = xvec(j+1);
      Kmat(j,j+1) =  dt/dfunk(x,t)* (cfunk(x,t)/dx^2 + (dcdxfunk(x,t) - bfunk(x,t))/(2*dx) );
   end
   % lower off-diagonal elements of matrix
   for j = ivaro:1:ivarf
      x = xvec(j+1);
      Kmat(j,j-1) =  dt/dfunk(x,t)* (cfunk(x,t)/dx^2 - (dcdxfunk(x,t) - bfunk(x,t))/(2*dx) );
   end
   
   function Rvec = getR(nx,ivaro,ivarf,t,xvec,dt,dx);
   Rvec = zeros(nx,1);
   for j = ivaro:1:ivarf
      x = xvec(j+1);
      Rvec(j) =  dt/dfunk(x,t)*ffunk(x,t);
   end
   
%
%  functions defining PDE
%

function a = afunk(x,t);
a = 0;

function b = bfunk(x,t);
b = 0;

function c = cfunk(x,t);
c = 1;

function dcdx = dcdxfunk(x,t);
dcdx = 0;

function d = dfunk(x,t);
d = 1;

function f = ffunk(x,t);
f = 000;

%
%  function defining initial condition
%

function ic = icfunk(x);
ic = 300;

%
%  functions defining LHS boundary condition
%

function f = aBCo(t);
f = 1;

function f = bBCo(t);
f = 0;

function f = cBCo(t);
f = -400;

%
%  functions defining RHS boundary condition
%

function f = aBCf(t);
f = 0;

function f = bBCf(t);
f = 1;

function f = cBCf(t);
f = 0;






