function [xvec,tvec,Tmat] = linparapde_euler
%
%  The routine linparapde_euler will solve 
%  a single linear 1-D parabolic PDE of the form
%
%  d(x,t)*dT/dt = div(c(x,t)*grad(T)) - a(x,t)*T -b(x,t)*dTdx +f(x,t)
%
%  using the Euler Method.
%
%  The initial condition must have the form
%  IC:  T(x,to) = To(x);
%
%  The boundary conditions must both be
%  Dirichlet Boundary Conditions of the form
%
%  BC 1:  T(xo,t) = T1(t);
%  BC 2:  T(xf,t) = T2(t);
%
%  The necessary functions
%  a(x,t), b(x,t), c(x,t), dcdx(x,t), f(x,t), To(x), T1(t) & T2(t)
%  are entered at the bottom of the file
%
%  The initial value, final value and discretization in time, t, 
%  to, tf, and dt must be entered at the top of the file.
%
%  The initial value, final value and discretization in space, x,
%  xo, xf, and dx must be entered at the top of the file.
%
%  sample execution:
%  [xvec,tvec,Tmat] = linparapde_euler;
%  code written by:  David J. Keffer
%
%  University of Tennessee, dkeffer@utk.edu
%  code written:  October 21, 2001
%  comments last updated:  February 26, 2014
%
clear all;
close all;
% discretize time
to = 0;
tf = 1.0e-0;
dt = 1.0e-3;
tvec = [to:dt:tf];
nt = length(tvec);

% discretize space
xo = 0;
xf = 1.0;
dx = 0.1;
xvec = [xo:dx:xf];
nx = length(xvec);

% dimension solution matrix
Tmat = zeros(nx,nt);

% dimension temporary vectors
nvar = nx-2;
Told = zeros(nvar,1);
Tnew = zeros(nvar,1);

% apply initial conditions
for i = 1:1:nx
	x = xvec(i);
	Tmat(i,1) = icfunk(x);
end

% apply boundary conditions
for i = 2:1:nt
	t = tvec(i);
	Tmat(1,i) = bc1funk(t);
	Tmat(nx,i) = bc2funk(t);
end

% loop over times
Told = Tmat(2:1:nx-1,1);
for i = 2:1:nt
   t = tvec(i);
   Kmat = zeros(nvar,nvar);
   Rvec = zeros(nvar,1);
   % diagonal elements of matrix
   for j = 1:1:nvar
      x = xvec(j+1);
      Kmat(j,j) =  dt/dfunk(x,t)* (-2*cfunk(x,t)/dx^2 - afunk(x,t) );
   end
   % upper off-diagonal elements of matrix
   for j = 1:1:nvar-1
      x = xvec(j+1);
      Kmat(j,j+1) =  dt/dfunk(x,t)* (cfunk(x,t)/dx^2 + (dcdxfunk(x,t) - bfunk(x,t))/(2*dx) );
   end
   % lower off-diagonal elements of matrix
   for j = 2:1:nvar
      x = xvec(j+1);
      Kmat(j,j-1) =  dt/dfunk(x,t)* (cfunk(x,t)/dx^2 - (dcdxfunk(x,t) - bfunk(x,t))/(2*dx) );
   end
   % vector of constants
   for j = 1:1:nvar
      x = xvec(j+1);
      Rvec(j) =  dt/dfunk(x,t)*ffunk(x,t);
   end
   % incorporate BCs
   x = xvec(1);
   term = dt/dfunk(x,t)* (cfunk(x,t)/dx^2 - (dcdxfunk(x,t) - bfunk(x,t))/(2*dx) );
   Rvec(1) = Rvec(1) + term*bc1funk(t);
   x = xvec(nx);
   term = dt/dfunk(x,t)* (cfunk(x,t)/dx^2 + (dcdxfunk(x,t) - bfunk(x,t))/(2*dx) );
   Rvec(nvar) = Rvec(nvar) + term*bc2funk(t);
   Tnew = Told + Kmat*Told + Rvec;
   Tmat(2:1:nx-1,i) = Tnew;
   Told = Tnew;
end
% plot
figure(1);
nskip = 10;
for i = 1:nskip:nt
   plot(xvec,Tmat(:,i),'k-');
   %pause(1);
   hold on;
end
xlabel('position (m)')
ylabel('Temperature (K)');


function a = afunk(x,t);
a = 0;

function b = bfunk(x,t);
b = 0;

function c = cfunk(x,t);
c = 1;

function dcdx = dcdxfunk(x,t);
dcdx = 0;

function d = dfunk(x,t);
d = 1;

function f = ffunk(x,t);
f = 000;

function ic = icfunk(x);
ic = 300;

function bc1 = bc1funk(t);
bc1 = 300;

function bc2 = bc2funk(t);
bc2 = 400;



