function [xvec,tvec,Tmat] = parapde_1_anyBC_flow
%
%  The routine parapde_1_anyBC will solve 
%  a single non-linear 1-D parabolic PDE of the form
%
%  dT/dt = f(x,t,T,dTdx,d2Tdx2)
%
%  where the convection term dominates, 
%  using a second order method.
%
%  The initial condition must have the form
%  IC:  T(x,to) = To(x);
%
%  The boundary conditions can be
%  Dirichlet, Neumann or Mixed of the form
%  BC 1:  aBCo(t)*T(xo,t) + bBCo(t)*dTdx(xo,t) + cBCo(t) = 0;
%  BC 2:  aBCf(t)*T(xf,t) + bBCf(t)*dTdx(xf,t) + cBCf(t) = 0;
%
%  The necessary functions
%  pdefunk(x,t,T,dTdx,d2Tdx2), To(x),
%  aBCo(t), bBCo(t), cBCo(t), aBCf(t), bBCf(t), cBCf(t) 
%  are entered at the bottom of the file
%
%  The initial value, final value and discretization in time, t, 
%  to, tf, and dt must be entered at the top of the file.
%
%  The initial value, final value and discretization in space, x,
%  xo, xf, and dx must be entered at the top of the file.
%
%  The type of boundary conditions 'D', 'N' or 'M'
%  for each boundary must be entered at the top of the file.
%
%  sample execution:
%  [xvec,tvec,Tmat] = parapde_1_anyBC;
%
%  code written by:  David J. Keffer
%  University of Tennessee, dkeffer@utk.edu
%  code written:  October 21, 2001
%  comments last updated:  February 26, 2014
%
clear all;
close all;
%
%  define type of boundary condition
%  Choices are 'D' = Dirichlet, i.e. bBC(t) = 0
%  Choices are 'N' = Neumann, i.e. aBC(t) = 0
%  Choices are 'M' = Mixed, bBC(t) ~= 0 & aBC(t) ~= 0
%
BC(1) = 'D';
BC(2) = 'N';

% discretize time
to = 0;
tf = 2.0e+3;
dt = 1.0e+0;
tvec = [to:dt:tf];
nt = length(tvec);

% discretize space
xo = 0;
xf = 10.0;
dx = 1.0e-0;
% include imaginary boundary nodes 
xvec = [xo-dx:dx:xf+dx];
nx = length(xvec);
dxi = 1.0/dx;

% dimension solution
Tmat = zeros(nx,nt);


% dimension temporary vectors
Told = zeros(nx,1);
Ttem = zeros(nx,1);
Tnew = zeros(nx,1);

% apply initial conditions to all real nodes
i = 1;
t = tvec(i);
for j = 2:1:nx-1
	Tmat(j,i) = icfunk(xvec(j));
end

% apply Neumann/Mixed BCs as ICs at imaginary nodes
if (BC(1) ~= 'D')
   %Tmat(1,i) =  2.0*dx/bBCo(t)*( aBCo(t)*Tmat(2,i)    + bBCo(t)/(2.0*dx)*Tmat(3,i)    + cBCo(t)); 
   Tmat(1,i) =       dx/bBCo(t)*( aBCo(t)*Tmat(2,i)    + bBCo(t)/(    dx)*Tmat(2,i)    + cBCo(t)); 
end
if (BC(2) ~= 'D')
   %Tmat(nx,i) = 2.0*dx/bBCf(t)*(-aBCf(t)*Tmat(nx-1,i) + bBCf(t)/(2.0*dx)*Tmat(nx-2,i) - cBCf(t));
    Tmat(nx,i) =     dx/bBCf(t)*(-aBCf(t)*Tmat(nx-1,i) + bBCf(t)/(    dx)*Tmat(nx-1,i) - cBCf(t));
end

%
%  Determine, based on type of BC, how to define which nodes are determined by PDE vs BCs
%  ivaro = index of first node to be solved by PDE 
%  ivarf = index of last node to be solved by PDE 
%  nvar = number of nodes to be solved by PDE
%
if (BC(1) == 'D') 
   ivaro = 3;
else
   ivaro = 2;
end
if (BC(2) == 'D') 
   ivarf = nx-2;
else
   ivarf = nx-1;
end
nvar = ivarf - ivaro + 1;

% loop over times
Told(1:nx) = Tmat(1:nx,i);
for i = 2:1:nt
%  update time
   told = tvec(i-1);
   t = tvec(i);

%
%  Prediction Step
%

%  compute first and second spatial derivatives
   for j = ivaro:1:ivarf
       %dTdx(j) = 0.5*( Told(j+1) - Told(j-1) )*dxi;
       dTdx(j) = ( Told(j) - Told(j-1) )*dxi;
       d2Tdx2(j) = ( Told(j+1) - 2.0*Told(j) + Told(j-1) )*dxi^2;
   end

% estimate slope  at beginning of temporal interval 
   for j = ivaro:1:ivarf
       k1(j) = pdefunk(xvec(j),tvec(i),Told(j),dTdx(j),d2Tdx2(j));
   end

% apply Euler method for the prediction step
   for j = ivaro:1:ivarf
       Ttem(j) = Told(j) + dt*k1(j);
   end

%  apply BCs at the prediction step
   if (BC(1) == 'D') 
      Ttem(2) = -cBCo(t)/aBCo(t);
   else
      %Ttem(1) =  2.0*dx/bBCo(t)*( aBCo(t)*Ttem(2)    + bBCo(t)/(2.0*dx)*Ttem(3)    + cBCo(t)); 
      Ttem(1) =       dx/bBCo(t)*( aBCo(t)*Ttem(2)    + bBCo(t)/(    dx)*Ttem(2)    + cBCo(t)); 
   end
   if (BC(2) == 'D') 
      Ttem(nx-1) = -cBCf(t)/aBCf(t);
   else
      %Ttem(nx) = 2.0*dx/bBCf(t)*(-aBCf(t)*Ttem(nx-1) + bBCf(t)/(2.0*dx)*Ttem(nx-2) - cBCf(t));  
      Ttem(nx) = dx/bBCf(t)*(-aBCf(t)*Ttem(nx-1) + bBCf(t)/(dx)*Ttem(nx-1) - cBCf(t));  
   end
   
%
%  Correction Step
%

%  compute first and second spatial derivatives
   for j = ivaro:1:ivarf
       %dTdx(j) = 0.5*( Ttem(j+1) - Ttem(j-1) )*dxi;
       dTdx(j) =     ( Ttem(j)   - Ttem(j-1) )*dxi;
       d2Tdx2(j) = ( Ttem(j+1) - 2.0*Ttem(j) + Ttem(j-1) )*dxi^2;
   end

% estimate slope  at end of temporal interval 
   for j = ivaro:1:ivarf
       k2(j) = pdefunk(xvec(j),tvec(i),Ttem(j),dTdx(j),d2Tdx2(j));
   end

% apply second-order method for the correction step
   for j = ivaro:1:ivarf
       Tnew(j) = Told(j) + 0.50*dt*(k1(j)+k2(j));
   end

%  apply BCs at the correction step
   if (BC(1) == 'D') 
      Tnew(2) = -cBCo(t)/aBCo(t);
   else
      %Tnew(1) =  2.0*dx/bBCo(t)*( aBCo(t)*Tnew(2)    + bBCo(t)/(2.0*dx)*Tnew(3)    + cBCo(t)); 
      Tnew(1) =       dx/bBCo(t)*( aBCo(t)*Tnew(2)    + bBCo(t)/(    dx)*Tnew(2)    + cBCo(t)); 
   end
   if (BC(2) == 'D') 
      Tnew(nx-1) = -cBCf(t)/aBCf(t);
   else
      %Tnew(nx) = 2.0*dx/bBCf(t)*(-aBCf(t)*Tnew(nx-1) + bBCf(t)/(2.0*dx)*Tnew(nx-2) - cBCf(t));  
      Tnew(nx) = dx/bBCf(t)*(-aBCf(t)*Tnew(nx-1) + bBCf(t)/(dx)*Tnew(nx-1) - cBCf(t));  
   end
   
% store new temperatures
   Tmat(1:nx,i) = Tnew(1:nx);
   Told(1:nx) = Tnew(1:nx);
end

% plot
figure(1);
nskip = 10;
for i = 1:nskip:nt
   plot(xvec(2:nx-1),Tmat(2:nx-1,i),'k-');
   %pause(1/5);
   hold on;
end
xlabel('position (m)')
ylabel('Concentration (mol/m^3)');


%
%  functions defining PDE
%

function dydt = pdefunk(x,t,y,dydx,d2ydx2);
% molar concentrations [mol/m^3]
CA = y(1);
% velocity [m/s]
v = 0.1;
% diffusivity [m^2/s]
D = 1.0e-9;
%  rate constant [m^6/mol^2/s]
k = 1.0e-5;
% stoichiometric coefficients
nuA = -2.0;
% reaction rate [mol/m^3/s]
rate = k*CA^2;
dydt = -v*dydx + D*d2ydx2 + nuA*rate;

%
%  function defining initial condition
%

function ic = icfunk(x);
ic = 0;

%
%  functions defining LHS boundary condition
%

function f = aBCo(t);
f = 1;

function f = bBCo(t);
f = 0;

function f = cBCo(t);
f = -1000;

%
%  functions defining RHS boundary condition
%

function f = aBCf(t);
f = 0;


function f = bBCf(t);
f = 1;

function f = cBCf(t);
f = 0;



