function [xvec,yvec,tvec,Tmat] = parapde_n_anyBC_2d
%
%  The routine parapde_n_anyBC will solve 
%  a system of non-linear 2-D parabolic PDEs of the form
%
%  dT/dt = f(x,y,t,T,dTdx,dTdy,d2Tdx2,d2Ydy2,d2Tdxdy)
%
%  using a second order method
%
%  The initial conditions must have the form
%  IC:  T(x,y,to,k_eq) = To(x,y,k_eq);
%
%  The boundary conditions can be
%  Dirichlet, Neumann or Mixed of the form
%  x BC 1:  aBCxo(y,t,k)*T(xo,y,t) + bBCxo(y,t,k)*dTdx(xo,y,t) + cBCxo(y,t,k) = 0;
%  x BC 2:  aBCxf(y,t,k)*T(xf,y,t) + bBCxf(y,t,k)*dTdx(xf,y,t) + cBCxf(y,t,k) = 0;
%  y BC 3:  aBCyo(x,t,k)*T(x,yo,t) + bBCyo(x,t,k)*dTdx(x,yo,t) + cBCyo(x,t,k) = 0;
%  y BC 4:  aBCyf(x,t,k)*T(x,yf,t) + bBCyf(x,t,k)*dTdx(x,yf,t) + cBCyf(x,t,k) = 0;
%
%  The necessary functions:
%  pdefunkf(x,y,t,T,dTdx,dTdy,d2Tdx2,d2Ydy2,d2Tdxdy),
%  To(x,y,k),
%  aBCxo(y,t,k), bBCxo(y,t,k), cBCxo(y,t,k), aBCxf(y,t,k), bBCxf(y,t,k), cBCxf(y,t,k) 
%  aBCyo(x,t,k), bBCyo(x,t,k), cBCyo(x,t,k), aBCyf(x,t,k), bBCyf(x,t,k), cBCyf(x,t,k) 
%  are entered at the bottom of the file
%
%  The number of equations, neq, must be entered at the top of the file 
%
%  The initial value, final value and discretization in time, t, 
%  to, tf, and dt must be entered at the top of the file.
%
%  The initial value, final value and discretization in space, x,
%  xo, xf, and dx must be entered at the top of the file.
%
%  The initial value, final value and discretization in space, y,
%  yo, yf, and dy must be entered at the top of the file.
%
%  The type of boundary conditions 'D', 'N' or 'M'
%  for each boundary must be entered at the top of the file.
%
%  sample execution:
%  [xvec,yvec,tvec,Tmat] = parapde_n_anyBC_2d_example01;
%
%  code written by:  David J. Keffer
%  University of Tennessee, dkeffer@utk.edu
%  code written:  October 21, 2001
%  comments last updated:  February 12, 2015
%
clear all;
close all;
%  define number of PDEs
neq = 1;

%
%  define type of boundary condition
%  Choices are 'D' = Dirichlet, i.e. bBC(t) = 0
%  Choices are 'N' = Neumann, i.e. aBC(t) = 0
%  Choices are 'M' = Mixed, bBC(t) ~= 0 & aBC(t) ~= 0
%
%  The dimension is the first index 1 = z, 2 = r
%  The boundary is the second index  1 = o, 2 = f 
%  The equation is the third index.  from 1 no neq
BC(1,1,1) = 'D';
BC(1,2,1) = 'D';
BC(2,1,1) = 'N';
BC(2,2,1) = 'N';


% discretize time
to = 0;
tf = 1.0e+2;
dt = 1.0e-1;
tvec = [to:dt:tf];
nt = length(tvec);

% discretize the first spatial dimension
xo = 0;
xf = 1.0;
dx = 5.0e-2;
% include imaginary boundary nodes 
xvec = [xo-dx:dx:xf+dx];
nx = length(xvec);
dxi = 1.0/dx;

% discretize the second spatial dimension
dy = 1.0e-2;
yo = dy;
yf = 0.1;
% include imaginary boundary nodes 
yvec = [yo-dy:dy:yf+dy];
ny = length(yvec);
dyi = 1.0/dy;

% dimension solution
Tmat = zeros(nx,ny,nt,neq);


% dimension temporary vectors
Told = zeros(nx,ny,neq);
Ttem = zeros(nx,ny,neq);
Tnew = zeros(nx,ny,neq);

% apply initial conditions to all real nodes
i = 1;
t = tvec(i);
for k = 1:1:neq
   for j = 2:1:nx-1
      for m = 2:1:ny-1
	     Tmat(j,m,i,k) = icfunk(xvec(j),yvec(m),k); 
      end
   end
end


%
% apply Neumann/Mixed BCs as ICs at imaginary nodes
%
for k = 1:1:neq
%  along initial boundary of first spatial dimension
   if (BC(1,1,k) ~= 'D')
      for m = 2:1:ny-1
         y = yvec(m);
         Tmat(1,m,i,k)  = 2.0*dx/bBCxo(y,t,k)*( aBCxo(y,t,k)*Tmat(2,m,i,k)    + bBCxo(y,t,k)/(2.0*dx)*Tmat(3,m,i,k)    + cBCxo(y,t,k)); 
      end
   end
%  along second boundary of first spatial dimension
   if (BC(1,2,k) ~= 'D')
      for m = 2:1:ny-1
         y = yvec(m);
         Tmat(nx,m,i,k) = 2.0*dx/bBCxf(y,t,k)*(-aBCxf(y,t,k)*Tmat(nx-1,m,i,k) + bBCxf(y,t,k)/(2.0*dx)*Tmat(nx-2,m,i,k) - cBCxf(y,t,k));
      end
   end
%  along initial boundary of second spatial dimension
   if (BC(2,1,k) ~= 'D')
      for j = 2:1:nx-1
         x = xvec(j);
         Tmat(j,1,i,k)  = 2.0*dy/bBCyo(x,t,k)*( aBCyo(x,t,k)*Tmat(j,2,i,k)    + bBCyo(x,t,k)/(2.0*dy)*Tmat(j,3,i,k)    + cBCyo(x,t,k)); 
      end
   end
%  along second boundary of second spatial dimension
   if (BC(2,2,k) ~= 'D')
      for j = 2:1:nx-1
         x = xvec(j);
         Tmat(j,ny,i,k) = 2.0*dy/bBCyf(x,t,k)*(-aBCyf(x,t,k)*Tmat(j,ny-1,i,k) + bBCyf(x,t,k)/(2.0*dy)*Tmat(j,ny-2,i,k) - cBCyf(x,t,k));
      end
   end
end
%
%  Determine, based on type of BC, how to define which nodes are determined by PDE vs BCs
%  ivaro = index of first node to be solved by PDE 
%  ivarf = index of last node to be solved by PDE 
%  nvar = number of nodes to be solved by PDE
%
%  first index of these three variables is spatial dimension
%  second index is equation 
ivaro = zeros(2,neq);
ivarf = zeros(2,neq);
nvar = zeros(2,neq);
for k = 1:1:neq
%  initial boundary of first spatial dimension 
   if (BC(1,1,k) == 'D') 
      ivaro(1,k) = 3;
   else
      ivaro(1,k) = 2;
   end
%  final boundary of first spatial dimension 
   if (BC(1,2,k) == 'D') 
      ivarf(1,k) = nx-2;
   else
      ivarf(1,k) = nx-1;
   end
%  number of nodes along first spatial dimension
   nvar(1,k) = ivarf(1,k) - ivaro(1,k) + 1;
%  initial boundary of second spatial dimension 
   if (BC(2,1,k) == 'D') 
      ivaro(2,k) = 3;
   else
      ivaro(2,k) = 2;
   end
%  final boundary of second spatial dimension 
   if (BC(2,2,k) == 'D') 
      ivarf(2,k) = ny-2;
   else
      ivarf(2,k) = ny-1;
   end
%  number of nodes along second spatial dimension
   nvar(2,k) = ivarf(2,k) - ivaro(2,k) + 1;
end


% loop over times
Told(1:nx,1:ny,1:neq) = Tmat(1:nx,1:ny,i,1:neq);
for i = 2:1:nt
%  update time
   t = tvec(i);

%
%  Prediction Step
%

% allocate memory for spatial derivatives
dTdx = zeros(nx,ny,neq);
dTdy = zeros(nx,ny,neq);
d2Tdx2 = zeros(nx,ny,neq);
d2Tdy2 = zeros(nx,ny,neq);
d2Tdxdy = zeros(nx,ny,neq);

%  compute first and second spatial derivatives
   for k = 1:1:neq
      for j = ivaro(1,k):1:ivarf(1,k)
         for m = ivaro(2,k):1:ivarf(2,k)
            dTdx(j,m,k) = 0.5*( Told(j+1,m,k) - Told(j-1,m,k) )*dxi;
            dTdy(j,m,k) = 0.5*( Told(j,m+1,k) - Told(j,m-1,k) )*dyi;
            d2Tdx2(j,m,k) = ( Told(j+1,m,k) - 2.0*Told(j,m,k) + Told(j-1,m,k) )*dxi^2;
            d2Tdy2(j,m,k) = ( Told(j,m+1,k) - 2.0*Told(j,m,k) + Told(j,m-1,k) )*dyi^2;
            d2Tdxdy(j,m,k) = ( Told(j+1,m+1,k) - Told(j+1,m-1,k) - Told(j-1,m+1,k) + Told(j-1,m-1,k) )*dxi*dyi*0.25;
         end
      end
   end
   
k1 = zeros(nx,ny,neq);   
% estimate slope  at beginning of temporal interval 
   for k = 1:1:neq
      for j = ivaro(1,k):1:ivarf(1,k)
         for m = ivaro(2,k):1:ivarf(2,k)
%             fprintf(1,'main %e %e %e %e %e\n', k, j, m, ivaro(2,k), ivarf(2,k));
            k1(j,m,k) = pdefunk(xvec(j),yvec(m),tvec(i),Told(j,m,1:neq),dTdx(j,m,1:neq),dTdy(j,m,1:neq),d2Tdx2(j,m,1:neq),d2Tdy2(j,m,1:neq),d2Tdxdy(j,m,1:neq),k);
         end
      end
   end

% apply Euler method for the prediction step
   for k = 1:1:neq
      for j = ivaro(1,k):1:ivarf(1,k)
         for m = ivaro(2,k):1:ivarf(2,k)
            Ttem(j,m,k) = Told(j,m,k) + dt*k1(j,m,k);
         end
      end
   end

%  apply BCs at the prediction step
   for k = 1:1:neq
%  along initial boundary of first spatial dimension
      if (BC(1,1,k) == 'D')
         for m = 2:1:ny-1
            y = yvec(m);
            Ttem(2,m,k) = -cBCxo(y,t,k)/aBCxo(y,t,k);
         end
      else
         for m = 2:1:ny-1
            y = yvec(m);
            Ttem(1,m,k)  = 2.0*dx/bBCxo(y,t,k)*( aBCxo(y,t,k)*Ttem(2,m,k)    + bBCxo(y,t,k)/(2.0*dx)*Ttem(3,m,k)    + cBCxo(y,t,k)); 
         end
      end
%  along second boundary of first spatial dimension
      if (BC(1,2,k) == 'D')
         for m = 2:1:ny-1
            y = yvec(m);
            Ttem(nx-1,m,k) = -cBCxf(y,t,k)/aBCxf(y,t,k);
         end
      else
         for m = 2:1:ny-1
            y = yvec(m);
            Ttem(nx,m,k) = 2.0*dx/bBCxf(y,t,k)*(-aBCxf(y,t,k)*Ttem(nx-1,m,k) + bBCxf(y,t,k)/(2.0*dx)*Ttem(nx-2,m,k) - cBCxf(y,t,k));
         end
      end
%  along initial boundary of second spatial dimension
     if (BC(2,1,k) == 'D')
         for j = 2:1:nx-1
            x = xvec(j);
            Ttem(j,2,k) = -cBCyo(x,t,k)/aBCyo(x,t,k);
         end
     else
         for j = 2:1:nx-1
            x = xvec(j);
            Ttem(j,1,k)  = 2.0*dy/bBCyo(x,t,k)*( aBCyo(x,t,k)*Ttem(j,2,k)    + bBCyo(x,t,k)/(2.0*dy)*Ttem(j,3,k)    + cBCyo(x,t,k)); 
         end
      end
%  along second boundary of second spatial dimension
      if (BC(2,2,k) == 'D')
         for j = 2:1:nx-1
            x = xvec(j);
            Ttem(j,ny-1,k) = -cBCyf(x,t,k)/aBCyf(x,t,k);
         end
     else
         for j = 2:1:nx-1
            x = xvec(j);
            Ttem(j,ny,k) = 2.0*dy/bBCyf(x,t,k)*(-aBCyf(x,t,k)*Ttem(j,ny-1,k) + bBCyf(x,t,k)/(2.0*dy)*Ttem(j,ny-2,k) - cBCyf(x,t,k));
         end
      end
   end
   
%
%  Correction Step
%

%  compute first and second spatial derivatives
   for k = 1:1:neq
      for j = ivaro(1,k):1:ivarf(1,k)
         for m = ivaro(2,k):1:ivarf(2,k)
            dTdx(j,m,k) = 0.5*( Ttem(j+1,m,k) - Ttem(j-1,m,k) )*dxi;
            dTdy(j,m,k) = 0.5*( Ttem(j,m+1,k) - Ttem(j,m-1,k) )*dyi;
            d2Tdx2(j,m,k) = ( Ttem(j+1,m,k) - 2.0*Ttem(j,m,k) + Ttem(j-1,m,k) )*dxi^2;
            d2Tdy2(j,m,k) = ( Ttem(j,m+1,k) - 2.0*Ttem(j,m,k) + Ttem(j,m-1,k) )*dyi^2;
            d2Tdxdy(j,m,k) = ( Ttem(j+1,m+1,k) - Ttem(j+1,m-1,k) - Ttem(j-1,m+1,k) + Ttem(j-1,m-1,k) )*dxi*dyi*0.25;
         end
      end
   end
   
k2 = zeros(nx,ny,neq);  
% estimate slope  at end of temporal interval 
   for k = 1:1:neq
      for j = ivaro(1,k):1:ivarf(1,k)
         for m = ivaro(2,k):1:ivarf(2,k)
            k2(j,m,k) = pdefunk(xvec(j),yvec(m),tvec(i),Ttem(j,m,1:neq),dTdx(j,m,1:neq),dTdy(j,m,1:neq),d2Tdx2(j,m,1:neq),d2Tdy2(j,m,1:neq),d2Tdxdy(j,m,1:neq),k);
         end
      end
   end

% apply second-order method for the correction step
   for k = 1:1:neq
      for j = ivaro(1,k):1:ivarf(1,k)
         for m = ivaro(2,k):1:ivarf(2,k)
            Tnew(j,m,k) = Told(j,m,k) + 0.50*dt*(k1(j,m,k)+k2(j,m,k));
         end
      end
   end


%  apply BCs at the correction step
   for k = 1:1:neq
%  along initial boundary of first spatial dimension
      if (BC(1,1,k) == 'D')
         for m = 2:1:ny-1
            y = yvec(m);
            Tnew(2,m,k) = -cBCxo(y,t,k)/aBCxo(y,t,k);
         end
      else
         for m = 2:1:ny-1
            y = yvec(m);
            Tnew(1,m,k)  = 2.0*dx/bBCxo(y,t,k)*( aBCxo(y,t,k)*Tnew(2,m,k)    + bBCxo(y,t,k)/(2.0*dx)*Tnew(3,m,k)    + cBCxo(y,t,k)); 
         end
      end
%  along second boundary of first spatial dimension
      if (BC(1,2,k) == 'D')
         for m = 2:1:ny-1
            y = yvec(m);
            Tnew(nx-1,m,k) = -cBCxf(y,t,k)/aBCxf(y,t,k);
         end
      else
         for m = 2:1:ny-1
            y = yvec(m);
            Tnew(nx,m,k) = 2.0*dx/bBCxf(y,t,k)*(-aBCxf(y,t,k)*Tnew(nx-1,m,k) + bBCxf(y,t,k)/(2.0*dx)*Tnew(nx-2,m,k) - cBCxf(y,t,k));
         end
      end
%  along initial boundary of second spatial dimension
     if (BC(2,1,k) == 'D')
         for j = 2:1:nx-1
            x = xvec(j);
            Tnew(j,2,k) = -cBCyo(x,t,k)/aBCyo(x,t,k);
         end
     else
         for j = 2:1:nx-1
            x = xvec(j);
            Tnew(j,1,k)  = 2.0*dy/bBCyo(x,t,k)*( aBCyo(x,t,k)*Tnew(j,2,k)    + bBCyo(x,t,k)/(2.0*dy)*Tnew(j,3,k)    + cBCyo(x,t,k)); 
         end
      end
%  along second boundary of second spatial dimension
      if (BC(2,2,k) == 'D')
         for j = 2:1:nx-1
            x = xvec(j);
            Tnew(j,ny-1,k) = -cBCyf(x,t,k)/aBCyf(x,t,k);
         end
     else
         for j = 2:1:nx-1
            x = xvec(j);
            Tnew(j,ny,k) = 2.0*dy/bBCyf(x,t,k)*(-aBCyf(x,t,k)*Tnew(j,ny-1,k) + bBCyf(x,t,k)/(2.0*dy)*Tnew(j,ny-2,k) - cBCyf(x,t,k));
         end
      end
   end

   
% store new temperatures
   Tmat(1:nx,1:ny,i,1:neq) = Tnew(1:nx,1:ny,1:neq);
   Told(1:nx,1:ny,1:neq) = Tnew(1:nx,1:ny,1:neq);
end

% plot via movie

% number of frames in movie
nframe = 40;
% frames per second
fps = 3;
if (nt > nframe)
	nskip = round(nt/nframe);
else
   nskip = 1;
end
lplot = 1;
if (lplot == 1)
   fpsinv = 1.0/fps;
   newplot;
   [xp,yp] = meshgrid(xvec(2:1:nx-1),yvec(2:1:ny-1));
   for k = 1:1:neq
      Tmax(k)=max(max(max(Tmat(2:nx-1,2:ny-1,1:nt,k))));
      Tmin(k)=min(min(min(Tmat(2:nx-1,2:ny-1,1:nt,k))));
      ztext(k) = Tmin(k) + 1.4*(Tmax(k)-Tmin(k));
   end
   xtext = xo + 0.01*(xf-xo);
   ytext = yo + 0.99*(yf-yo);
   for i=1:nskip:nt
       temps = char(strcat('time = ', num2str(tvec(i)), ' sec '));
       for k = 1:1:neq
          fig = figure(k);
          set(fig, 'Units','normalized');
          set(fig,'Position', [.08+(k-1)*.44 .2 .4 .6]);
		  %color_index = get_plot_color(k);
          surf(xp, yp, Tmat(2:1:nx-1,2:1:ny-1,i,k)')
          axis([xo xf yo yf Tmin(k) Tmax(k)]);
          xlabel('first spatial dimension'); 
          ylabel('second spatial dimension'); 
          zlabel('dependent variable');
	     %legend (int2str([1:neq]'));
          text(xtext, ytext,ztext(k), temps);
          hold off
       end
       pause(fpsinv);
	end
end


%
%  functions defining PDE
%

function dTdt_out = pdefunk(z,r,t,Told,dTdz,dTdr,d2Tdz2,d2Tdr2,d2Tdzdr,keq);
%
%  physical properties
%
% rho = density [kg/m^3]
rho = 8960.0;
% Cp = heat capacity [J/kg/K]
Cp = 384.6;
% k = thermal conductivity [W/m/K]
k = 401.0;
%  alpha = thermal diffusivity
alpha = k/rho/Cp;
%  Stefan-Boltzmann constant [J/s/m^2/K^4]
sigma = 5.670373e-8; 
% gray body permittivity
eps = 0.15; % (for dull Cu)
%eps = 0.0; % no radiative loss
% plate geometry
length = 1.0; % [m]
radius = 0.1; % [m]
pi = 2.0*asin(1.0);
area = 2*pi*radius*length;
volume = pi*radius*radius*length; 
s = area/volume; % 1/m 
% surrounding temperature [K]
Tsurround = 300.0; 
%
% PDE
%
dTdt_out = alpha*(d2Tdz2(1) + 1.0/r*dTdr(1) + d2Tdr2(1)) - eps*sigma*s/(rho*Cp)*(Told(1)^4 - Tsurround^4);
%fprintf(1,'%e %e %e %e \n',z, r, t, dTdt_out);



%
%  function defining initial condition
%

function ic_out = icfunk(x,y,keq);
ic(1) = 1000.0;
ic_out = ic(keq);
  
%
%  functions defining initial boundary condition in first spatial dimension
%

function fout = aBCxo(y,t,k);
f(1) = 1;
fout = f(k);

function fout = bBCxo(y,t,k);
f(1) = 0;
fout = f(k);

function fout = cBCxo(y,t,k);
f(1) = -800;
fout = f(k);

%
%  functions defining final boundary condition in first spatial dimension
%

function fout = aBCxf(y,t,k);
f(1) = 1;
fout = f(k);

function fout = bBCxf(y,t,k);
f(1) = 0;
fout = f(k);

function fout = cBCxf(y,t,k);
f(1) = -1000;
fout = f(k);

%
%  functions defining initial boundary condition in second spatial dimension
%

function fout = aBCyo(x,t,k);
f(1) = 0;
fout = f(k);

function fout = bBCyo(x,t,k);
f(1) = 1;
fout = f(k);

function fout = cBCyo(x,t,k);
f(1) = 0;
fout = f(k);

%
%  functions defining final boundary condition in second spatial dimension
%

function fout = aBCyf(x,t,k);
%  heat transfer coefficient in [W/m^2/K]
h = 4.0e+1;
f(1) = h;
fout = f(k);

function fout = bBCyf(x,t,k);
% kc = thermal conductivity [W/m/K]
kc = 401.0;
f(1) = kc;
fout = f(k);

function fout = cBCyf(x,t,k);
%  heat transfer coefficient in [W/m^2/K]
h = 4.0e+1;
% surrounding temperature [K]
Tsurround = 300.0; 
f(1) = -h*Tsurround;
fout = f(k);

























%
% this little function sets colors for curves in plot
%
function color_index = get_plot_color(i);
if (i == 1) 
   color_index = 'k-';
elseif (i == 2)
   color_index = 'r-';
elseif (i == 3)
   color_index = 'b-';
elseif (i == 4)
   color_index = 'g-';
elseif (i == 5)
   color_index = 'm-';
elseif (i == 6) 
   color_index = 'k:';
elseif (i == 7)
   color_index = 'r:';
elseif (i == 8)
   color_index = 'b:';
elseif (i == 9)
   color_index = 'g:';
elseif (i == 10)
   color_index = 'm:';
else
   color_index = 'k-';
end
